@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2023 Joël Krähemann
+ * Copyright (C) 2005-2025 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -1101,32 +1101,27 @@ GList*
 ags_automation_find_near_timestamp(GList *automation, guint line,
 				   AgsTimestamp *timestamp)
 {
-  AgsTimestamp *current_timestamp;
+  GList *bisect_start, *bisect_end, *bisect_center;
+  GList *bisect_match;
 
-  GList *retval;
-  GList *current_start, *current_end, *current;
-
-  guint current_line;
-  guint64 current_x, x;
-  guint length, position;
+  guint64 x;
+  guint length;
   gboolean use_ags_offset;
-  gboolean success;
+  gboolean has_more;
 
   if(automation == NULL){
     return(NULL);
   }
 
-  current_start = automation;
-  current_end = g_list_last(automation);
+  bisect_start = automation;
+  bisect_end = g_list_last(automation);
   
   length = g_list_length(automation);
-  position = (length - 1) / 2;
 
-  current = g_list_nth(current_start,
-		       position);
+  bisect_center = g_list_nth(bisect_start,
+			     length / 2);
 
-  x = 0;
-  use_ags_offset = FALSE;
+  use_ags_offset = TRUE;
   
   if(ags_timestamp_test_flags(timestamp,
 			      AGS_TIMESTAMP_OFFSET)){
@@ -1138,201 +1133,248 @@ ags_automation_find_near_timestamp(GList *automation, guint line,
     x = ags_timestamp_get_unix_time(timestamp);
 
     use_ags_offset = FALSE;
+  }else{
+    return(NULL);
   }
   
-  retval = NULL;
-  success = FALSE;
+  bisect_match = NULL;
+  has_more = TRUE;
   
-  while(!success && current != NULL){
-    current_x = 0;
-    current_line = 0;
+  while(has_more &&
+	bisect_start != NULL &&
+	bisect_end != NULL &&
+	bisect_center != NULL){
+    AgsTimestamp *current_timestamp;
+
+    GList *current_match;
+    
+    guint bisect_start_line, bisect_end_line, bisect_center_line;
+    guint64 bisect_start_x, bisect_end_x, bisect_center_x;
+    
+    gboolean bisect_head;
+
+    current_match = NULL;
+
+    bisect_head = TRUE;
     
     /* check current - start */
-    if(current_start != NULL){
-      g_object_get(current_start->data,
-		   "line", &current_line,
-		   NULL);
-    }else{
-      break;
-    }
+    bisect_start_line = ags_automation_get_line((AgsAutomation *) bisect_start->data);
+
+    current_timestamp = ags_automation_get_timestamp((AgsAutomation *) bisect_start->data);
+
+    bisect_start_x = 0;
     
-    if(current_line == line){
-      if(timestamp == NULL){
-	retval = current_start;
-
-	break;
-      }
-
-      g_object_get(current_start->data,
-		   "timestamp", &current_timestamp,
-		   NULL);
-
-      if(current_timestamp != NULL){
-	if(use_ags_offset){
-	  current_x = ags_timestamp_get_ags_offset(current_timestamp);
-	  
-	  g_object_unref(current_timestamp);
-	    
-	  if(current_x > x){
-	    break;
-	  }
-	}else{
-	  current_x = ags_timestamp_get_unix_time(current_timestamp);
-	  
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x > x){
-	    break;
-	  }
-	}
-
-	if(use_ags_offset){
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
-	    retval = current_start;
-	    
-	    break;
-	  }
-	}else{
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
-	    retval = current_start;
-	    
-	    break;
-	  }
-	}
+    if(current_timestamp != NULL){
+      if(use_ags_offset){
+	bisect_start_x = ags_timestamp_get_ags_offset(current_timestamp);
       }else{
-	g_warning("inconsistent data");
+	bisect_start_x = ags_timestamp_get_unix_time(current_timestamp);
       }
+
+      g_object_unref(current_timestamp);
+    }else{
+      g_warning("inconsistent data");
     }
 
+    if(ags_automation_get_line((AgsAutomation *) bisect_start->data) == line){
+      if(timestamp == NULL){
+	current_match = bisect_start;
+      }
+    }
+   
     /* check current - end */
-    g_object_get(current_end->data,
-		 "line", &current_line,
-		 NULL);
+    bisect_end_line = ags_automation_get_line((AgsAutomation *) bisect_end->data);
+
+    current_timestamp = ags_automation_get_timestamp((AgsAutomation *) bisect_end->data);
+
+    bisect_end_x = 0;
     
-    if(current_line == line){
-      if(timestamp == NULL){
-	retval = current_end;
-	
-	break;
-      }
-
-      g_object_get(current_end->data,
-		   "timestamp", &current_timestamp,
-		   NULL);
-
-      if(current_timestamp != NULL){
-	if(use_ags_offset){
-	  current_x = ags_timestamp_get_ags_offset(current_timestamp);
-	  
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x < x){
-	    break;
-	  }
-	}else{
-	  current_x = ags_timestamp_get_unix_time(current_timestamp);
-	  
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x < x){
-	    break;
-	  }
-	}
-
-	if(use_ags_offset){
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
-	    retval = current_end;
-	    
-	    break;
-	  }
-	}else{
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
-	    retval = current_end;
-	    
-	    break;
-	  }
-	}
+    if(current_timestamp != NULL){
+      if(use_ags_offset){
+	bisect_end_x = ags_timestamp_get_ags_offset(current_timestamp);
       }else{
-	g_warning("inconsistent data");
+	bisect_end_x = ags_timestamp_get_unix_time(current_timestamp);
       }
-    }
 
-    /* check current - center */
-    g_object_get(current->data,
-		 "line", &current_line,
-		 NULL);
-    
-    if(current_line == line){
-      if(timestamp == NULL){
-	retval = current;
-	
-	break;
-      }
-    
-      g_object_get(current->data,
-		   "timestamp", &current_timestamp,
-		   NULL);
-
-      if(current_timestamp != NULL){
-	if(use_ags_offset){
-	  current_x = ags_timestamp_get_ags_offset(current_timestamp);
-
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_OFFSET &&
-	     current_line == line){
-	    retval = current;
-	    
-	    break;
-	  }
-	}else{
-	  current_x = ags_timestamp_get_unix_time(current_timestamp);
-
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_DURATION &&
-	     current_line == line){
-	    retval = current;
-	    
-	    break;
-	  }
-	}
-      }else{
-	g_warning("inconsistent data");
-      }
-    }
-    
-    if(length <= 3){
-      break;
-    }
-
-    if(current_x < x){
-      current_start = current->next;
-      current_end = current_end->prev;
-    }else if(current_x > x){
-      current_start = current_start->next;
-      current_end = current->prev;
+      g_object_unref(current_timestamp);
     }else{
-      current_start = current_start->next;
-      //NOTE:JK: we want progression
-      //current_end = current_end->prev;
+      g_warning("inconsistent data");
+    }
+    
+    /* check current - center */
+    bisect_center_line = ags_automation_get_line((AgsAutomation *) bisect_center->data);
+
+    current_timestamp = ags_automation_get_timestamp((AgsAutomation *) bisect_center->data);
+
+    bisect_center_x = 0;
+    
+    if(current_timestamp != NULL){
+      if(use_ags_offset){
+	bisect_center_x = ags_timestamp_get_ags_offset(current_timestamp);
+      }else{
+	bisect_center_x = ags_timestamp_get_unix_time(current_timestamp);
+      }
+
+      g_object_unref(current_timestamp);
+    }else{
+      g_warning("inconsistent data");
+    }
+    
+    /* check x */    	
+    if(use_ags_offset){
+      if(bisect_end_x >= x &&
+	 bisect_end_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
+	if(bisect_end_line == line){
+	  current_match = bisect_end;
+	}
+
+	bisect_head = FALSE;
+      }
+    }else{
+      if(bisect_end_x >= x &&
+	 bisect_end_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
+	if(bisect_end_line == line){
+	  current_match = bisect_end;
+	}
+	
+	bisect_head = FALSE;
+      }
+    }
+	
+    if(use_ags_offset){
+      if(bisect_center_x >= x &&
+	 bisect_center_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
+	if(bisect_center_line == line){
+	  current_match = bisect_center;
+	}
+
+	bisect_head = TRUE;
+      }
+    }else{
+      if(bisect_center_x >= x &&
+	 bisect_center_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
+	if(bisect_center_line == line){
+	  current_match = bisect_center;
+	}
+	
+	bisect_head = TRUE;
+      }
+    }
+    
+    if(use_ags_offset){
+      if(bisect_start_x >= x &&
+	 bisect_start_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
+	if(bisect_start_line == line){
+	  current_match = bisect_start;
+	}
+	
+	bisect_head = TRUE;
+      }
+    }else{
+      if(bisect_start_x >= x &&
+	 bisect_start_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
+	if(bisect_start_line == line){
+	  current_match = bisect_start;
+	}
+	
+	bisect_head = TRUE;
+      }
     }
 
-    length = g_list_position(current_start,
-			     current_end) + 1;
-    position = (length - 1) / 2;
+    if(bisect_start_x == bisect_center_x){
+      if(bisect_center_line < line){
+	bisect_head = FALSE;
+      }
+    }
 
-    current = g_list_nth(current_start,
-			 position);
+    if(current_match != NULL){
+      AgsTimestamp *match_timestamp;
+      
+      guint64 match_x, current_x;
+      
+      if(bisect_match != NULL){
+	current_timestamp = ags_automation_get_timestamp(current_match->data);
+	match_timestamp = ags_automation_get_timestamp(bisect_match->data);
+
+	match_x = 0;
+	current_x = 0;
+	
+	if(use_ags_offset){
+	  match_x = ags_timestamp_get_ags_offset(match_timestamp);
+	}else{
+	  match_x = ags_timestamp_get_unix_time(match_timestamp);
+	}
+
+	if(use_ags_offset){
+	  current_x = ags_timestamp_get_ags_offset(current_timestamp);
+	}else{
+	  current_x = ags_timestamp_get_unix_time(current_timestamp);
+	}
+	
+	if(current_x < match_x ||
+	   (current_x == match_x &&
+	    ags_automation_get_channel_type((AgsAutomation *) current_match->data) < ags_automation_get_channel_type((AgsAutomation *) bisect_match->data)) ||
+	   (current_x == match_x &&
+	    ags_automation_get_channel_type((AgsAutomation *) current_match->data) == ags_automation_get_channel_type((AgsAutomation *) bisect_match->data) &&
+	    g_strcmp0(AGS_AUTOMATION(current_match->data)->control_name, AGS_AUTOMATION(bisect_match->data)->control_name) < 0)){
+	  bisect_match = current_match;
+	}
+
+	if(current_timestamp != NULL){
+	  g_object_unref(current_timestamp);
+	}
+	
+	if(match_timestamp != NULL){
+	  g_object_unref(match_timestamp);
+	}
+      }else{
+	bisect_match = current_match;
+      }
+    }
+    
+    /* iterate */
+    if(length <= 3){
+      has_more = FALSE;
+    }
+
+    if(has_more){
+      if(bisect_head){
+	bisect_start = bisect_start->next;
+	bisect_end = bisect_center->prev;
+
+	length = 0;
+
+	if(bisect_start != NULL &&
+	   bisect_end != NULL){
+	  length = g_list_position(bisect_start,
+				   bisect_end);
+
+	  length++;
+	}
+	
+	bisect_center = g_list_nth(bisect_start,
+				   length / 2);
+      }else{
+	bisect_start = bisect_center->next;
+	bisect_end = bisect_end->prev;
+
+	length = 0;
+	
+	if(bisect_start != NULL &&
+	   bisect_end != NULL){
+	  length = g_list_position(bisect_start,
+				   bisect_end);
+
+	  length++;
+	}
+
+	bisect_center = g_list_nth(bisect_start,
+				   length / 2);
+      }
+    }
   }
 
-  return(retval);
+  return(bisect_match);
 }
 
 /**
@@ -1354,37 +1396,28 @@ ags_automation_find_near_timestamp_extended(GList *automation, guint line,
 					    GType channel_type, gchar *control_name,
 					    AgsTimestamp *timestamp)
 {
-  AgsTimestamp *current_timestamp;
+  GList *bisect_start, *bisect_end, *bisect_center;
+  GList *bisect_match;
 
-  GList *retval;
-  GList *current_start, *current_end, *current;
-
-  GType current_channel_type;
-
-  gchar *current_control_name;
-
-  guint current_line;
-  guint64 current_x, x;
-  guint length, position;
+  guint64 x;
+  guint length;
   gboolean use_ags_offset;
-  gboolean success;
+  gboolean has_more;
 
   if(automation == NULL){
     return(NULL);
   }
 
-  current_start = automation;
-  current_end = g_list_last(automation);
+  bisect_start = automation;
+  bisect_end = g_list_last(automation);
   
   length = g_list_length(automation);
-  position = (guint) floor((double) (length - 1) / 2.0);
 
-  current = g_list_nth(current_start,
-		       position);
-
-  x = 0;
-  use_ags_offset = FALSE;
+  bisect_center = g_list_nth(bisect_start,
+			     length / 2);
 
+  use_ags_offset = TRUE;
+  
   if(ags_timestamp_test_flags(timestamp,
 			      AGS_TIMESTAMP_OFFSET)){
     x = ags_timestamp_get_ags_offset(timestamp);
@@ -1395,231 +1428,264 @@ ags_automation_find_near_timestamp_extended(GList *automation, guint line,
     x = ags_timestamp_get_unix_time(timestamp);
 
     use_ags_offset = FALSE;
+  }else{
+    return(NULL);
   }
   
-  retval = NULL;
-  success = FALSE;
+  bisect_match = NULL;
+  has_more = TRUE;
   
-  while(!success && current != NULL){
-    current_x = 0;
+  while(has_more &&
+	bisect_start != NULL &&
+	bisect_end != NULL &&
+	bisect_center != NULL){
+    AgsTimestamp *current_timestamp;
 
+    GList *current_match;
+    
+    guint bisect_start_line, bisect_end_line, bisect_center_line;
+    guint64 bisect_start_x, bisect_end_x, bisect_center_x;
+    
+    gboolean bisect_head;
+
+    current_match = NULL;
+
+    bisect_head = TRUE;
+    
     /* check current - start */
-    g_object_get(current_start->data,
-		 "line", &current_line,
-		 "channel-type", &current_channel_type,
-		 "control-name", &current_control_name,
-		 NULL);
+    bisect_start_line = ags_automation_get_line((AgsAutomation *) bisect_start->data);
+
+    current_timestamp = ags_automation_get_timestamp((AgsAutomation *) bisect_start->data);
+
+    bisect_start_x = 0;
     
-    if(current_line == line &&
-       current_channel_type == channel_type &&
-       !g_strcmp0(current_control_name,
-		  control_name)){
-      g_free(current_control_name);
-      
-      if(timestamp == NULL){
-	retval = current_start;
-	
-	break;
-      }
-
-      g_object_get(current_start->data,
-		   "timestamp", &current_timestamp,
-		   NULL);
-
-      if(current_timestamp != NULL){
-	if(use_ags_offset){
-	  current_x = ags_timestamp_get_ags_offset(current_timestamp);
-	  
-	  g_object_unref(current_timestamp);
-	    
-	  if(current_x > x){
-	    break;
-	  }
-	}else{
-	  current_x = ags_timestamp_get_unix_time(current_timestamp);
-	  
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x > x){
-	    break;
-	  }
-	}
-
-	if(use_ags_offset){
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
-	    retval = current_start;
-	    
-	    break;
-	  }
-	}else{
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
-	    retval = current_start;
-	    
-	    break;
-	  }
-	}
+    if(current_timestamp != NULL){
+      if(use_ags_offset){
+	bisect_start_x = ags_timestamp_get_ags_offset(current_timestamp);
       }else{
-	g_warning("inconsistent data");
+	bisect_start_x = ags_timestamp_get_unix_time(current_timestamp);
       }
+
+      g_object_unref(current_timestamp);
     }else{
-      g_free(current_control_name);
+      g_warning("inconsistent data");
     }
 
+    if(bisect_start_line == line &&
+       !g_strcmp0(control_name, AGS_AUTOMATION(bisect_start->data)->control_name) &&
+       ags_automation_get_channel_type((AgsAutomation *) bisect_start->data) == channel_type){
+      if(timestamp == NULL){
+	current_match = bisect_start;
+      }
+    }
+   
     /* check current - end */
-    g_object_get(current_end->data,
-		 "line", &current_line,
-		 "channel-type", &current_channel_type,
-		 "control-name", &current_control_name,
-		 NULL);
+    bisect_end_line = ags_automation_get_line((AgsAutomation *) bisect_end->data);
+
+    current_timestamp = ags_automation_get_timestamp((AgsAutomation *) bisect_end->data);
+
+    bisect_end_x = 0;
     
-    if(current_line == line &&
-       current_channel_type == channel_type &&
-       !g_strcmp0(current_control_name,
-		  control_name)){
-      g_free(current_control_name);
-      
-      if(timestamp == NULL){
-	retval = current_end;
-	
-	break;
-      }
-
-      g_object_get(current_end->data,
-		   "timestamp", &current_timestamp,
-		   NULL);
-
-      if(current_timestamp != NULL){
-	if(use_ags_offset){
-	  current_x = ags_timestamp_get_ags_offset(current_timestamp);
-	  
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x < x){
-	    break;
-	  }
-	}else{
-	  current_x = ags_timestamp_get_unix_time(current_timestamp);
-	  
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x < x){
-	    break;
-	  }
-	}
-
-	if(use_ags_offset){
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
-	    retval = current_end;
-	    
-	    break;
-	  }
-	}else{
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
-	    retval = current_end;
-	    
-	    break;
-	  }
-	}
+    if(current_timestamp != NULL){
+      if(use_ags_offset){
+	bisect_end_x = ags_timestamp_get_ags_offset(current_timestamp);
       }else{
-	g_warning("inconsistent data");
+	bisect_end_x = ags_timestamp_get_unix_time(current_timestamp);
       }
-    }else{
-      g_free(current_control_name);
-    }
 
+      g_object_unref(current_timestamp);
+    }else{
+      g_warning("inconsistent data");
+    }
+    
     /* check current - center */
-    current_x = 0;
-    
-    g_object_get(current->data,
-		 "line", &current_line,
-		 "channel-type", &current_channel_type,
-		 "control-name", &current_control_name,
-		 NULL);
-    
-    if(current_line == line &&
-       current_channel_type == channel_type &&
-       !g_strcmp0(current_control_name,
-		  control_name)){
-      if(timestamp == NULL){
-	retval = current;
+    bisect_center_line = ags_automation_get_line((AgsAutomation *) bisect_center->data);
 
-	g_free(current_control_name);
-	
-	break;
+    current_timestamp = ags_automation_get_timestamp((AgsAutomation *) bisect_center->data);
+
+    bisect_center_x = 0;
+    
+    if(current_timestamp != NULL){
+      if(use_ags_offset){
+	bisect_center_x = ags_timestamp_get_ags_offset(current_timestamp);
+      }else{
+	bisect_center_x = ags_timestamp_get_unix_time(current_timestamp);
       }
-    
-      g_object_get(current->data,
-		   "timestamp", &current_timestamp,
-		   NULL);
 
-      if(current_timestamp != NULL){
+      g_object_unref(current_timestamp);
+    }else{
+      g_warning("inconsistent data");
+    }
+    
+    /* check x */    	
+    if(use_ags_offset){
+      if(bisect_end_x >= x &&
+	 bisect_end_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
+	if(bisect_end_line == line &&
+	   !g_strcmp0(control_name, AGS_AUTOMATION(bisect_end->data)->control_name) &&
+	   ags_automation_get_channel_type((AgsAutomation *) bisect_end->data) == channel_type){
+	  current_match = bisect_end;
+	}
+
+	bisect_head = FALSE;
+      }
+    }else{
+      if(bisect_end_x >= x &&
+	 bisect_end_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
+	if(bisect_end_line == line &&
+	   !g_strcmp0(control_name, AGS_AUTOMATION(bisect_end->data)->control_name) &&
+	   ags_automation_get_channel_type((AgsAutomation *) bisect_end->data) == channel_type){
+	  current_match = bisect_end;
+	}
+	
+	bisect_head = FALSE;
+      }
+    }
+	
+    if(use_ags_offset){
+      if(bisect_center_x >= x &&
+	 bisect_center_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
+	if(bisect_center_line == line &&
+	   !g_strcmp0(control_name, AGS_AUTOMATION(bisect_center->data)->control_name) &&
+	   ags_automation_get_channel_type((AgsAutomation *) bisect_center->data) == channel_type){
+	  current_match = bisect_center;
+	}
+
+	bisect_head = TRUE;
+      }
+    }else{
+      if(bisect_center_x >= x &&
+	 bisect_center_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
+	if(bisect_center_line == line &&
+	   !g_strcmp0(control_name, AGS_AUTOMATION(bisect_center->data)->control_name) &&
+	   ags_automation_get_channel_type((AgsAutomation *) bisect_center->data) == channel_type){
+	  current_match = bisect_center;
+	}
+	
+	bisect_head = TRUE;
+      }
+    }
+    
+    if(use_ags_offset){
+      if(bisect_start_x >= x &&
+	 bisect_start_x < x + AGS_AUTOMATION_DEFAULT_OFFSET){
+	if(bisect_start_line == line &&
+	   !g_strcmp0(control_name, AGS_AUTOMATION(bisect_start->data)->control_name) &&
+	   ags_automation_get_channel_type((AgsAutomation *) bisect_start->data) == channel_type){
+	  current_match = bisect_start;
+	}
+	
+	bisect_head = TRUE;
+      }
+    }else{
+      if(bisect_start_x >= x &&
+	 bisect_start_x < x + AGS_AUTOMATION_DEFAULT_DURATION){
+	if(bisect_start_line == line &&
+	   !g_strcmp0(control_name, AGS_AUTOMATION(bisect_start->data)->control_name) &&
+	   ags_automation_get_channel_type((AgsAutomation *) bisect_start->data) == channel_type){
+	  current_match = bisect_start;
+	}
+	
+	bisect_head = TRUE;
+      }
+    }
+    
+    if(bisect_start_x == bisect_center_x){
+      if(ags_automation_get_line((AgsAutomation *) bisect_center->data) < line){
+	bisect_head = FALSE;
+      }else if(bisect_center_line == line &&
+	       ags_automation_get_channel_type((AgsAutomation *) bisect_center->data) < channel_type){
+	bisect_head = FALSE;
+      }else if(bisect_center_line == line &&
+	       ags_automation_get_channel_type((AgsAutomation *) bisect_center->data) == channel_type &&
+	       g_strcmp0(control_name, AGS_AUTOMATION(bisect_center->data)->control_name) < 0){
+	bisect_head = FALSE;
+      }
+    }
+    
+    if(current_match != NULL){
+      AgsTimestamp *match_timestamp;
+      
+      guint64 match_x, current_x;
+      
+      if(bisect_match != NULL){
+	current_timestamp = ags_automation_get_timestamp(current_match->data);
+	match_timestamp = ags_automation_get_timestamp(bisect_match->data);
+
+	match_x = 0;
+	current_x = 0;
+	
+	if(use_ags_offset){
+	  match_x = ags_timestamp_get_ags_offset(match_timestamp);
+	}else{
+	  match_x = ags_timestamp_get_unix_time(match_timestamp);
+	}
+
 	if(use_ags_offset){
 	  current_x = ags_timestamp_get_ags_offset(current_timestamp);
-
-	  g_object_unref(current_timestamp);
-	  
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_OFFSET &&
-	     current_line == line &&
-	     current_channel_type == channel_type &&
-	     !g_strcmp0(current_control_name,
-			control_name)){
-	    retval = current;
-	    
-	    break;
-	  }
 	}else{
 	  current_x = ags_timestamp_get_unix_time(current_timestamp);
+	}
+	
+	if(current_x < match_x){
+	  bisect_match = current_match;
+	}
 
+	if(current_timestamp != NULL){
 	  g_object_unref(current_timestamp);
-	  
-	  if(current_x >= x &&
-	     current_x < x + AGS_AUTOMATION_DEFAULT_DURATION &&
-	     current_line == line &&
-	     current_channel_type == channel_type &&
-	     !g_strcmp0(current_control_name,
-			control_name)){
-	    retval = current;
-	    
-	    break;
-	  }
+	}
+	
+	if(match_timestamp != NULL){
+	  g_object_unref(match_timestamp);
 	}
       }else{
-	g_warning("inconsistent data");
+	bisect_match = current_match;
       }
+    }
     
-      g_free(current_control_name);
-    }
-      
-    if(position == 0){
-      break;
+    /* iterate */
+    if(length <= 3){
+      has_more = FALSE;
     }
 
-    if(current_x < x){
-      current_start = current->next;
-      current_end = current_end->prev;
-    }else if(current_x > x){
-      current_start = current_start->next;
-      current_end = current->prev;
-    }else{
-      current_start = current_start->next;
-      //NOTE:JK: we want progression
-      //current_end = current_end->prev;
+    if(has_more){
+      if(bisect_head){
+	bisect_start = bisect_start->next;
+	bisect_end = bisect_center->prev;
+
+	length = 0;
+
+	if(bisect_start != NULL &&
+	   bisect_end != NULL){
+	  length = g_list_position(bisect_start,
+				   bisect_end);
+
+	  length++;
+	}
+	
+	bisect_center = g_list_nth(bisect_start,
+				   length / 2);
+      }else{
+	bisect_start = bisect_center->next;
+	bisect_end = bisect_end->prev;
+
+	length = 0;
+	
+	if(bisect_start != NULL &&
+	   bisect_end != NULL){
+	  length = g_list_position(bisect_start,
+				   bisect_end);
+
+	  length++;
+	}
+
+	bisect_center = g_list_nth(bisect_start,
+				   length / 2);
+      }
     }
-
-    length = g_list_position(current_start,
-			     current_end) + 1;
-    position = (guint) floor((double) (length - 1) / 2.0);
-
-    current = g_list_nth(current_start,
-			 position);
   }
 
-  return(retval);
+  return(bisect_match);
 }
 
 /**
