@@ -45,9 +45,9 @@ const gchar *ags_fx_pattern_recycling_plugin_name = "ags-fx-pattern";
 GType
 ags_fx_pattern_recycling_get_type()
 {
-  static volatile gsize g_define_type_id__volatile = 0;
+  static gsize g_define_type_id__static = 0;
 
-  if(g_once_init_enter (&g_define_type_id__volatile)){
+  if(g_once_init_enter(&g_define_type_id__static)){
     GType ags_type_fx_pattern_recycling = 0;
 
     static const GTypeInfo ags_fx_pattern_recycling_info = {
@@ -67,10 +67,10 @@ ags_fx_pattern_recycling_get_type()
 							   &ags_fx_pattern_recycling_info,
 							   0);
 
-    g_once_init_leave(&g_define_type_id__volatile, ags_type_fx_pattern_recycling);
+    g_once_init_leave(&g_define_type_id__static, ags_type_fx_pattern_recycling);
   }
 
-  return g_define_type_id__volatile;
+  return(g_define_type_id__static);
 }
 
 void
