@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2023 Joël Krähemann
+ * Copyright (C) 2005-2025 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -27,6 +27,8 @@
 
 #include <ags/audio/ags_sound_enums.h>
 
+#include <ags/audio/thread/ags_audio_tree_dispatcher.h>
+
 G_BEGIN_DECLS
 
 #define AGS_TYPE_CHANNEL_THREAD                (ags_channel_thread_get_type())
@@ -84,6 +86,8 @@ struct _AgsChannelThread
   gboolean processing;
 
   AgsTaskLauncher *task_launcher;
+
+  AgsAudioTreeDispatcher *audio_tree_dispatcher;
 };
 
 struct _AgsChannelThreadClass
