@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2024 Joël Krähemann
+ * Copyright (C) 2005-2025 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -498,12 +498,14 @@ ags_level_realize(GtkWidget *widget)
   /* call parent */
   GTK_WIDGET_CLASS(ags_level_parent_class)->realize(widget);
 
+#if 0
   frame_clock = gtk_widget_get_frame_clock(widget);
   
   g_signal_connect(frame_clock, "update", 
 		   G_CALLBACK(ags_level_frame_clock_update_callback), widget);
 
   gdk_frame_clock_begin_updating(frame_clock);
+#endif
 }
 
 void
@@ -511,6 +513,7 @@ ags_level_unrealize(GtkWidget *widget)
 {
   GdkFrameClock *frame_clock;
 
+#if 0
   frame_clock = gtk_widget_get_frame_clock(widget);
   
   g_object_disconnect(frame_clock,
@@ -520,6 +523,7 @@ ags_level_unrealize(GtkWidget *widget)
 		      NULL);
 
   gdk_frame_clock_end_updating(frame_clock);
+#endif
   
   /* call parent */
   GTK_WIDGET_CLASS(ags_level_parent_class)->unrealize(widget);
@@ -539,33 +543,43 @@ ags_level_snapshot(GtkWidget *widget,
   GtkStyleContext *style_context;
 
   cairo_t *cr;
-
-  graphene_rect_t rect;
   
   int width, height;
   
-  style_context = gtk_widget_get_style_context((GtkWidget *) widget);  
-
   width = gtk_widget_get_width(widget);
   height = gtk_widget_get_height(widget);
   
-  graphene_rect_init(&rect,
-		     0.0, 0.0,
-		     (float) width, (float) height);
-  
   cr = gtk_snapshot_append_cairo(snapshot,
-				 &rect);
+				 &GRAPHENE_RECT_INIT (0, 0, width, height));
+  //  cairo_reference(cr);
+  
+  style_context = gtk_widget_get_style_context((GtkWidget *) widget);  
+
+  gtk_style_context_save(style_context);
   
   /* clear bg */
+#if 0
+  cairo_save(cr);
+  cairo_clip(cr);
+  
   gtk_render_background(style_context,
-			cr,
-			0.0, 0.0,
-			(gdouble) width, (gdouble) height);
+  			cr,
+  			0.0, 0.0,
+  			(gdouble) width, (gdouble) height);
 
+  cairo_restore(cr);
+#endif
+  
+  /* draw */
+  cairo_save(cr);
+  
   ags_level_draw((AgsLevel *) widget,
 		 cr,
 		 TRUE);
+
+  cairo_restore(cr);
   
+  gtk_style_context_restore(style_context);
   cairo_destroy(cr);
 }
 
@@ -1141,13 +1155,12 @@ ags_level_draw(AgsLevel *level,
   pango_cairo_show_layout(cr,
 			  layout);
 
-  g_object_unref(layout);
-
-  g_free(font_name);
-  g_free(text);
-  
   cairo_pop_group_to_source(cr);
   cairo_paint(cr);
+
+  g_object_unref(layout);
+
+  g_free(text);  
 }
 
 /**
